<?php 

class LoginDongleUpdater
{
    protected $option;
    
    /**
     * Retrieves the version of this plugin.
     * 
     * @return string
     */
    protected function version()
    {
        $plugin_data = get_plugin_data(LOGIN_DONGLE_FILENAME);
        $result = $plugin_data['Version'];
        return $result;
    }
    
    /**
     * Retrieves the version of the data of this plugin.
     * 
     * @return string
     */
    protected function dataVersion()
    {
        $sitedata = $this->option->keysGet(array('message', 'version'));
        if (isset($sitedata['message']))
        {
            return isset($sitedata['version']) ? $sitedata['version'] : '1';
        }
        return $this->version();
    }
    
    /**
     * Stores the version of the data of this plugin.
     * 
     * @return string
     */
    protected function dataVersionUpdate($version)
    {
        $this->option->keysSet(array('version' => $version));
    }
    
    /**
     * Moves the challenge/response from the site option to each user's.
     * 
     */
    protected function moveOptionsFromSiteToUser()
    {
        //up to version 1.2.2 user settings did not exist and admin's 'challenge' and 'response'
        //were stored among site settings into the login_dongle option, alongside 'message'
        
        $users = get_users(array('fields' => array('ID','user_login')));
        echo '<dl><dt>Fixing user options (up to ' . count($users) . ')...</dt><dd>';
        $sitedata = $this->option->get();
        if (! isset($sitedata['challenge']))
        {
            //no options to move
            echo 'nothing to do. (challenge not set)</dd></dl>';
            return;
        }
        echo '<ol>';
        $userdata['challenge'] = $sitedata['challenge'];
        $userdata['response']  = $sitedata['response'];
        foreach ($users as $user) 
        {
            echo '<li>for user ' . $user->user_login . '...';
            $this->option->set($userdata, $user->user_login);
            echo ' done</li>';
        }
        echo '</ol></dd></dl>';
        
        echo '<dl><dt>Fixing site options...</dt><dd>';
        unset($sitedata['challenge'], $sitedata['response']);
        $this->option->set($sitedata);
        echo 'done</dd></dl>';
    }
    
    /**
     * Updates to the given $checkVersion using the method with the given $update name.
     * 
     * @param string $checkVersion
     * @param string $update
     */
    protected function updateToVersion($checkVersion, $update)
    {
        $dataVersion = $this->dataVersion();
        if (Ando_Version::compare("$dataVersion < $checkVersion"))
        {
            if (is_callable(array($this, $update)))
            {
                $args = func_get_args();
                $args = array_splice($args, 2);
                call_user_func_array(array($this, $update), $args);
            }
            $this->dataVersionUpdate($checkVersion);
        }
    }
    
    /**
     * Updates anything from previous versions.
     * Sends an email to the admin with a report.
     * 
     * This method is a handler bound to the activation hook.
     */
    public function update()
    {
        require_once 'Ando/Version.php';
        require_once 'LoginDongleMail.php';
        ob_start(); 
        ///////////////////////////////////////////activation event handlers cannot output anything
        $dataVersion = $this->dataVersion(); //version before updating (usually an option)
        
        $this->updateToVersion('1.3.0', 'moveOptionsFromSiteToUser');
        
        $version = $this->version();
        $this->dataVersionUpdate($version);
        ////////////////////////////////////////////so I send everything by email to the site admin
        $output = ob_get_clean();
        if (Ando_Version::compare("$dataVersion < $version"))
        {
            $output = "<html><body>$output<h4>" . __('The Login Dongle plugin has been updated.', LoginDonglePlugin::OPTION_NAME) . "</h4></body></html>";
            LoginDongleMail::send(get_option('admin_email'), sprintf(__(' updated to %s|updated plugin, %s is the new version', LoginDonglePlugin::OPTION_NAME), $version), $output, 'text/html');
        }
    }
    
    public function __construct()
    {
        require_once 'LoginDongleOption.php';
        $this->option = new LoginDongleOption();
        
        register_activation_hook(LOGIN_DONGLE_FILENAME, array($this, 'update'));
    }
}
