<?php

class LoginDongleUserAdmin
{
    const MAXLEN_CHALLENGE = 20;
    const MAXLEN_ANSWER    = 20;
    
    /**
     * @var LoginDongleOption
     */
    protected $option;
    
    public function __construct()
    {
        require_once 'LoginDongleOption.php';
        $this->option = new LoginDongleOption();
        
        add_action('profile_personal_options', array($this, 'handleGET')); //owner view
        //add_action('personal_options_update', array($this, 'handlePOST')); //"
        
        add_action('edit_user_profile', array($this, 'handleGET'));        //admin view
        add_action('profile_update', array($this, 'handlePOST'));          //"
    }
    
	/**
     * Returns the default settings for the user option.
     * 
     * @return struct
     */
    protected function defaultSettings()
    {
        $hash = strtoupper(md5(time()));
        $result = array(
            'challenge' => substr($hash,  0, 5), 
            'response'  => substr($hash, -3, 3),
        );
        return $result;
    }
    
    /**
     * Returns the code for the bookmarklet, based on the given challenge.
     * 
     * @param struct $data
     * @return string
     */
    protected function bookmarklet( $data )
    {
    	$challenge = $data['challenge'];
    	if ('' == $challenge)
    	{
    	    return;
    	}
    	$fieldName = LoginDonglePlugin::challengeFieldName($challenge);
    	$loginUrl  = LoginDonglePlugin::scriptUrl(wp_login_url());
    	$loginPath = preg_replace('@^' . preg_quote($_SERVER['HTTP_HOST'], '@') . '/@', '', $loginUrl);
    	//the following JS is semi-minified, just remove double spaces and new lines to fully minify it
    	$result = <<< END_source
(function($){
  var form$=$('form[action*="://LOGIN_URL"]');
  if(form$.length!=1)
    form$=$('form[action^="/LOGIN_PATH"]');
  if(form$.length!=1)
    return alert('This login dongle is for LOGIN_URL.');
  var challenge='CHALLENGE';
  var response=$.trim(prompt(challenge,''));
  $('<input type="hidden">')
    .appendTo(form$)
    .attr('name','FIELD_NAME')
    .attr('value',response);
  $('[type="submit"]',form$)
    .click()
})(jQuery);
END_source;
        $result = str_replace('  ', '', $result);
        $result = preg_replace("@\r?\n@", '', $result);
        $result = preg_replace('@\bLOGIN_URL\b@',  addslashes($loginUrl),  $result);
    	$result = preg_replace('@\bLOGIN_PATH\b@', addslashes($loginPath), $result);
    	$result = preg_replace('@\bCHALLENGE\b@',  addslashes($challenge), $result);
        $result = preg_replace('@\bFIELD_NAME\b@', addslashes($fieldName), $result);
    	return $result;
    }
    
    protected function echo_toggle($class, $show, $hide)
    {
?>
<script type="text/javascript">
jQuery(function($){ 
	$('a.<?php echo $class; ?>').click(function(e) { 
		e.preventDefault();
		$('.<?php echo $class; ?>').toggle(); 
	}); 
});
</script>
<a class="<?php echo $class; ?>" href="#"><?php echo $show; ?></a>
<a class="<?php echo $class; ?>" href="#" style="display: none;"><?php echo $hide; ?></a>
<?php
    }
    
    /**
     * Echoes the html for the user settings page.
     * 
     * This method is a handler bound to the profile_personal_options and edit_user_profile action.
     * 
     * @param WP_User $user
     */
    public function handleGET($user)
    {
        $username = $user->user_login;
        $data = $this->option->get($username);
        if (! is_array($data))
        {
            //$data = $this->defaultSettings();
            $data = array(
                'challenge' => '',
                'response'  => '',
            );
        }
        if (! empty($data['challenge']))
        {
            $code = $this->bookmarklet($data);
            $dongle_pc = 'javascript:' . htmlspecialchars(rawurlencode($code), ENT_COMPAT); //safest
            $dongle_sp = 'javascript:' . htmlspecialchars($code, ENT_COMPAT);
            $link_pc = '<strong><a href="' . $dongle_pc . '" title="Log In">' 
                    . __('encoded|encoded bookmarlet', LoginDonglePlugin::OPTION_NAME) . '</a></strong>';
            $link_sp = '<strong><a href="' . $dongle_sp . '" title="Log In">' 
                    . __('raw|raw bookmarlet', LoginDonglePlugin::OPTION_NAME) . '</a></strong>';
        }
        
        $__send = __('Send on next update', LoginDonglePlugin::OPTION_NAME);
        $__show_code = __('Show code', LoginDonglePlugin::OPTION_NAME);
        $__hide_code = __('Hide code', LoginDonglePlugin::OPTION_NAME);
        $__show_help = __('Show help', LoginDonglePlugin::OPTION_NAME);
        $__hide_help = __('Hide help', LoginDonglePlugin::OPTION_NAME);
        
?>
<div style="margin: 2em 0;">
    <h3>Login Dongle</h3>
<p><?php _e(

'Login Dongle makes the login for your personal use only, by means of a conventional challenge &gt;&gt; response mechanism.'

, LoginDonglePlugin::OPTION_NAME); ?> | <?php $this->echo_toggle('logindongle_help', $__show_help, $__hide_help); ?></p>
<p class="logindongle_help" style="display: none;"><?php _e(

'The challenge and its expected response resemble passwords, but they are fundamentally different. 
This data is supposed to have enough complexity to deter the casual user of your PC, if any. 
So feel free to use simple pairs, but such that only you can easily associate the response to the challenge, 
while others will have a harder time. E.G. color &gt;&gt; ful. 
(For your security, please do not use this example literally!).'

, LoginDonglePlugin::OPTION_NAME); ?></p>
<p class="logindongle_help" style="display: none;"><?php _e(

'You yourself won\'t be able to log into your blog without your most current login dongle, 
unless you disable this plugin right from the file system, 
as described in the <a href="http://wordpress.org/extend/plugins/login-dongle/faq/" target="_blank">FAQ</a>.'

, LoginDonglePlugin::OPTION_NAME); ?></p>
<p class="logindongle_help" style="display: none;"><?php printf(__(

'To be able to log into your blog using a different browser or a different system in the future, 
check the <em>%s</em> option. After saving, you\'ll receive the bookmarklet code by email.|
%s is "Send on next update"'

, LoginDonglePlugin::OPTION_NAME), $__send); ?></p>
	<table class="form-table">
    	<tbody>
    		<tr valign="top">
    			<th scope="row"><?php _e('Challenge', LoginDonglePlugin::OPTION_NAME); ?></th>
    			<td>
    				<input type="text" id="challenge" 
						name="<?php echo LoginDonglePlugin::OPTION_NAME; ?>[challenge]" 
						value="<?php echo htmlspecialchars($data['challenge']); ?>"
						size="<?php echo (self::MAXLEN_CHALLENGE + 2); ?>" 
						maxlength="<?php echo self::MAXLEN_CHALLENGE; ?>"  
					/><br />
					<span class="description"><?php printf(__('1 to %s chars.', LoginDonglePlugin::OPTION_NAME), self::MAXLEN_CHALLENGE); ?> 
					<?php _e('Leave this empty to be able to log in without a login dongle.', LoginDonglePlugin::OPTION_NAME); ?></span>
				</td>
			</tr>
    		<tr valign="top">
    			<th scope="row"><?php _e('Response', LoginDonglePlugin::OPTION_NAME); ?></th>
    			<td>
    				<input type="text" id="response" 
						name="<?php echo LoginDonglePlugin::OPTION_NAME; ?>[response]" 
						value="<?php echo htmlspecialchars($data['response']); ?>"
						size="<?php echo (self::MAXLEN_ANSWER + 2); ?>" 
						maxlength="<?php echo self::MAXLEN_ANSWER; ?>"  
					/><br />
					<span class="description"><?php printf(__('1 to %s chars.', LoginDonglePlugin::OPTION_NAME), self::MAXLEN_ANSWER); ?></span>
				</td>
			</tr>
<?php 
        if ($code) //I don't show the bookmaklets if no challenge is set
        {
?>
    		<tr valign="top">
    			<th scope="row"><?php _e('Bookmarklet for loggin in', LoginDonglePlugin::OPTION_NAME); ?></th>
    			<td><input type="checkbox" name="login_dongle_send" /> <?php echo $__send; ?><br />
    			<?php printf(__(
    			
'Whenever you change the challenge or the response, after saving
remember to drag and drop into your bookmarks this %1$s or %2$s bookmarklet.'

    			, LoginDonglePlugin::OPTION_NAME), $link_sp, $link_pc); ?></td>
    		</tr>
			<tr valign="top">
    			<th scope="row" style="padding-left: 20px;"><?php _e('Raw source', LoginDonglePlugin::OPTION_NAME); ?></th>
    			<td>
    				<?php _e('better for SmartPhones', LoginDonglePlugin::OPTION_NAME); ?> | 
    				<?php $this->echo_toggle('code_sp', $__show_code, $__hide_code); ?><br />
    				<textarea class="code_sp" style="width:60%; display: none;"><?php echo $dongle_sp; ?></textarea>
				</td>
			</tr>
			<tr valign="top">
    			<th scope="row" style="padding-left: 20px;"><?php _e('Encoded source', LoginDonglePlugin::OPTION_NAME); ?></th>
    			<td>
    				<?php _e('better for PCs', LoginDonglePlugin::OPTION_NAME); ?> | 
    				<?php $this->echo_toggle('code_pc', $__show_code, $__hide_code); ?><br />
    				<textarea class="code_pc" style="width:60%; display: none;"><?php echo $dongle_pc; ?></textarea>
				</td>
			</tr>
<?php 
        }
?>
		</tbody>
	</table>
</div>
<?php
    }
    
    /**
     * Saves POST data coming from the user settings page.
     * 
     * This method is a handler bound to the profile_update action.
     * 
     * @param integer $user_id
     */
    public function handlePOST($user_id)
    {
        $data = stripslashes_deep($_POST[LoginDonglePlugin::OPTION_NAME]);
        $data['challenge'] = substr(trim($data['challenge']), 0, self::MAXLEN_CHALLENGE);
        $data['response']  = substr(trim($data['response']),  0, self::MAXLEN_ANSWER);
        
        $user = new WP_User($user_id);
        $username = $user->user_login;
        $email = $user->user_email;
        
        $this->option->keysSet($data, $username);
        if (! ($data['challenge'] && $_POST['login_dongle_send']))
        {
            return;
        }
        $code = $this->bookmarklet($data);
        require_once 'LoginDongleMail.php';
        LoginDongleMail::send($email, ': ' . sprintf(__("encoded version of %s's bookmarklet source (better for PCs)|%s is a username", LoginDonglePlugin::OPTION_NAME), $username), 'javascript:' . rawurlencode($code));
        LoginDongleMail::send($email, ': ' . sprintf(__("raw version of %s's bookmarklet source (better for SmartPhones)|%s is a username", LoginDonglePlugin::OPTION_NAME), $username), 'javascript:' . $code);
    }
}
